@@ -7,10 +7,9 @@ from isort.format import format_simplified
 
 from . import parse, sorting, wrap
 from .comments import add_to_line as with_comments
+from .identify import STATEMENT_DECLARATIONS
 from .settings import DEFAULT_CONFIG, Config
 
-STATEMENT_DECLARATIONS: Tuple[str, ...] = ("def ", "cdef ", "cpdef ", "class ", "@", "async def")
-
 
 def sorted_imports(
     parsed: parse.ParsedContent,
@@ -49,16 +48,32 @@ def sorted_imports(
     pending_lines_before = False
     for section in sections:
         straight_modules = parsed.imports[section]["straight"]
-        straight_modules = sorting.naturally(
-            straight_modules,
-            key=lambda key: sorting.module_key(
-                key, config, section_name=section, straight_import=True
-            ),
-        )
+        if not config.only_sections:
+            straight_modules = sorting.naturally(
+                straight_modules,
+                key=lambda key: sorting.module_key(
+                    key, config, section_name=section, straight_import=True
+                ),
+                reverse=config.reverse_sort,
+            )
+
         from_modules = parsed.imports[section]["from"]
-        from_modules = sorting.naturally(
-            from_modules, key=lambda key: sorting.module_key(key, config, section_name=section)
-        )
+        if not config.only_sections:
+            from_modules = sorting.naturally(
+                from_modules,
+                key=lambda key: sorting.module_key(key, config, section_name=section),
+                reverse=config.reverse_sort,
+            )
+
+            if config.star_first:
+                star_modules = []
+                other_modules = []
+                for module in from_modules:
+                    if "*" in parsed.imports[section]["from"][module]:
+                        star_modules.append(module)
+                    else:
+                        other_modules.append(module)
+                from_modules = star_modules + other_modules
 
         straight_imports = _with_straight_imports(
             parsed, config, straight_modules, section, remove_imports, import_type
@@ -89,17 +104,10 @@ def sorted_imports(
                     comments_above = []
                 else:
                     new_section_output.append(line)
-
+            # only_sections options is not imposed if force_sort_within_sections is True
             new_section_output = sorting.naturally(
                 new_section_output,
-                key=partial(
-                    sorting.section_key,
-                    order_by_type=config.order_by_type,
-                    force_to_top=config.force_to_top,
-                    lexicographical=config.lexicographical,
-                    length_sort=config.length_sort,
-                    reverse_relative=config.reverse_relative,
-                ),
+                key=partial(sorting.section_key, config=config),
             )
 
             # uncollapse comments
@@ -153,43 +161,46 @@ def sorted_imports(
         output_at = parsed.import_index
     formatted_output[output_at:0] = output
 
-    imports_tail = output_at + len(output)
-    while [
-        character.strip() for character in formatted_output[imports_tail : imports_tail + 1]
-    ] == [""]:
-        formatted_output.pop(imports_tail)
+    if output:
+        imports_tail = output_at + len(output)
+        while [
+            character.strip() for character in formatted_output[imports_tail : imports_tail + 1]
+        ] == [""]:
+            formatted_output.pop(imports_tail)
 
-    if len(formatted_output) > imports_tail:
-        next_construct = ""
-        tail = formatted_output[imports_tail:]
+        if len(formatted_output) > imports_tail:
+            next_construct = ""
+            tail = formatted_output[imports_tail:]
 
-        for index, line in enumerate(tail):
-            should_skip, in_quote, *_ = parse.skip_line(
-                line,
-                in_quote="",
-                index=len(formatted_output),
-                section_comments=config.section_comments,
-                needs_import=False,
-            )
-            if not should_skip and line.strip():
-                if (
-                    line.strip().startswith("#")
-                    and len(tail) > (index + 1)
-                    and tail[index + 1].strip()
-                ):
-                    continue
-                next_construct = line
-                break
-            elif in_quote:
-                next_construct = line
-                break
+            for index, line in enumerate(tail):
+                should_skip, in_quote, *_ = parse.skip_line(
+                    line,
+                    in_quote="",
+                    index=len(formatted_output),
+                    section_comments=config.section_comments,
+                    needs_import=False,
+                )
+                if not should_skip and line.strip():
+                    if (
+                        line.strip().startswith("#")
+                        and len(tail) > (index + 1)
+                        and tail[index + 1].strip()
+                    ):
+                        continue
+                    next_construct = line
+                    break
+                if in_quote:
+                    next_construct = line
+                    break
 
-        if config.lines_after_imports != -1:
-            formatted_output[imports_tail:0] = ["" for line in range(config.lines_after_imports)]
-        elif extension != "pyi" and next_construct.startswith(STATEMENT_DECLARATIONS):
-            formatted_output[imports_tail:0] = ["", ""]
-        else:
-            formatted_output[imports_tail:0] = [""]
+            if config.lines_after_imports != -1:
+                formatted_output[imports_tail:0] = [
+                    "" for line in range(config.lines_after_imports)
+                ]
+            elif extension != "pyi" and next_construct.startswith(STATEMENT_DECLARATIONS):
+                formatted_output[imports_tail:0] = ["", ""]
+            else:
+                formatted_output[imports_tail:0] = [""]
 
     if parsed.place_imports:
         new_out_lines = []
@@ -222,19 +233,20 @@ def _with_from_imports(
 
         import_start = f"from {module} {import_type} "
         from_imports = list(parsed.imports[section]["from"][module])
-        if not config.no_inline_sort or (
-            config.force_single_line and module not in config.single_line_exclusions
-        ):
-            ignore_case = config.force_alphabetical_sort_within_sections
+        if (
+            not config.no_inline_sort
+            or (config.force_single_line and module not in config.single_line_exclusions)
+        ) and not config.only_sections:
             from_imports = sorting.naturally(
                 from_imports,
                 key=lambda key: sorting.module_key(
                     key,
                     config,
                     True,
-                    ignore_case,
+                    config.force_alphabetical_sort_within_sections,
                     section_name=section,
                 ),
+                reverse=config.reverse_sort,
             )
         if remove_imports:
             from_imports = [
@@ -252,7 +264,8 @@ def _with_from_imports(
         if config.combine_as_imports and not ("*" in from_imports and config.combine_star):
             if not config.no_inline_sort:
                 for as_import in as_imports:
-                    as_imports[as_import] = sorting.naturally(as_imports[as_import])
+                    if not config.only_sections:
+                        as_imports[as_import] = sorting.naturally(as_imports[as_import])
             for from_import in copy.copy(from_imports):
                 if from_import in as_imports:
                     idx = from_imports.index(from_import)
@@ -312,22 +325,41 @@ def _with_from_imports(
                         from_comments = parsed.categorized_comments["straight"].get(
                             f"{module}.{from_import}"
                         )
-                        output.extend(
-                            with_comments(
-                                from_comments,
-                                wrap.line(import_start + as_import, parsed.line_separator, config),
-                                removed=config.ignore_comments,
-                                comment_prefix=config.comment_prefix,
+
+                        if not config.only_sections:
+                            output.extend(
+                                with_comments(
+                                    from_comments,
+                                    wrap.line(
+                                        import_start + as_import, parsed.line_separator, config
+                                    ),
+                                    removed=config.ignore_comments,
+                                    comment_prefix=config.comment_prefix,
+                                )
+                                for as_import in sorting.naturally(as_imports[from_import])
+                            )
+
+                        else:
+                            output.extend(
+                                with_comments(
+                                    from_comments,
+                                    wrap.line(
+                                        import_start + as_import, parsed.line_separator, config
+                                    ),
+                                    removed=config.ignore_comments,
+                                    comment_prefix=config.comment_prefix,
+                                )
+                                for as_import in as_imports[from_import]
                             )
-                            for as_import in sorting.naturally(as_imports[from_import])
-                        )
                     else:
                         output.append(wrap.line(single_import_line, parsed.line_separator, config))
                     comments = None
             else:
                 while from_imports and from_imports[0] in as_imports:
                     from_import = from_imports.pop(0)
-                    as_imports[from_import] = sorting.naturally(as_imports[from_import])
+
+                    if not config.only_sections:
+                        as_imports[from_import] = sorting.naturally(as_imports[from_import])
                     from_comments = (
                         parsed.categorized_comments["straight"].get(f"{module}.{from_import}") or []
                     )
@@ -397,18 +429,22 @@ def _with_from_imports(
                         parsed.categorized_comments["nested"].get(module, {}).pop(from_import, None)
                     )
                     if comment:
+                        from_imports.remove(from_import)
+                        if from_imports:
+                            use_comments = []
+                        else:
+                            use_comments = comments
+                            comments = None
                         single_import_line = with_comments(
-                            comments,
+                            use_comments,
                             import_start + from_import,
                             removed=config.ignore_comments,
                             comment_prefix=config.comment_prefix,
                         )
                         single_import_line += (
-                            f"{comments and ';' or config.comment_prefix} " f"{comment}"
+                            f"{use_comments and ';' or config.comment_prefix} " f"{comment}"
                         )
                         output.append(wrap.line(single_import_line, parsed.line_separator, config))
-                        from_imports.remove(from_import)
-                        comments = None
 
                 from_import_section = []
                 while from_imports and (
@@ -468,7 +504,13 @@ def _with_from_imports(
                             config=config,
                             multi_line_output=wrap.Modes.VERTICAL_GRID,  # type: ignore
                         )
-                        if max(len(x) for x in import_statement.split("\n")) > config.line_length:
+                        if (
+                            max(
+                                len(import_line)
+                                for import_line in import_statement.split(parsed.line_separator)
+                            )
+                            > config.line_length
+                        ):
                             import_statement = other_import_statement
                 if not do_multiline_reformat and len(import_statement) > config.line_length:
                     import_statement = wrap.line(import_statement, parsed.line_separator, config)
@@ -487,6 +529,40 @@ def _with_straight_imports(
     import_type: str,
 ) -> List[str]:
     output: List[str] = []
+
+    as_imports = any((module in parsed.as_map["straight"] for module in straight_modules))
+
+    # combine_straight_imports only works for bare imports, 'as' imports not included
+    if config.combine_straight_imports and not as_imports:
+        if not straight_modules:
+            return []
+
+        above_comments: List[str] = []
+        inline_comments: List[str] = []
+
+        for module in straight_modules:
+            if module in parsed.categorized_comments["above"]["straight"]:
+                above_comments.extend(parsed.categorized_comments["above"]["straight"].pop(module))
+            if module in parsed.categorized_comments["straight"]:
+                inline_comments.extend(parsed.categorized_comments["straight"][module])
+
+        combined_straight_imports = ", ".join(straight_modules)
+        if inline_comments:
+            combined_inline_comments = " ".join(inline_comments)
+        else:
+            combined_inline_comments = ""
+
+        output.extend(above_comments)
+
+        if combined_inline_comments:
+            output.append(
+                f"{import_type} {combined_straight_imports}  # {combined_inline_comments}"
+            )
+        else:
+            output.append(f"{import_type} {combined_straight_imports}")
+
+        return output
+
     for module in straight_modules:
         if module in remove_imports:
             continue
@@ -494,25 +570,25 @@ def _with_straight_imports(
         import_definition = []
         if module in parsed.as_map["straight"]:
             if parsed.imports[section]["straight"][module]:
-                import_definition.append(f"{import_type} {module}")
+                import_definition.append((f"{import_type} {module}", module))
             import_definition.extend(
-                f"{import_type} {module} as {as_import}"
+                (f"{import_type} {module} as {as_import}", f"{module} as {as_import}")
                 for as_import in parsed.as_map["straight"][module]
             )
         else:
-            import_definition.append(f"{import_type} {module}")
+            import_definition.append((f"{import_type} {module}", module))
 
         comments_above = parsed.categorized_comments["above"]["straight"].pop(module, None)
         if comments_above:
             output.extend(comments_above)
         output.extend(
             with_comments(
-                parsed.categorized_comments["straight"].get(module),
+                parsed.categorized_comments["straight"].get(imodule),
                 idef,
                 removed=config.ignore_comments,
                 comment_prefix=config.comment_prefix,
             )
-            for idef in import_definition
+            for idef, imodule in import_definition
         )
 
     return output
@@ -554,5 +630,4 @@ def _with_star_comments(parsed: parse.ParsedContent, module: str, comments: List
     star_comment = parsed.categorized_comments["nested"].get(module, {}).pop("*", None)
     if star_comment:
         return comments + [star_comment]
-    else:
-        return comments
+    return comments
