@@ -121,7 +121,9 @@ class _Config:
     py_version: str = "3"
     force_to_top: FrozenSet[str] = frozenset()
     skip: FrozenSet[str] = DEFAULT_SKIP
+    extend_skip: FrozenSet[str] = frozenset()
     skip_glob: FrozenSet[str] = frozenset()
+    extend_skip_glob: FrozenSet[str] = frozenset()
     skip_gitignore: bool = False
     line_length: int = 79
     wrap_length: int = 0
@@ -200,8 +202,17 @@ class _Config:
     dedup_headings: bool = False
     only_sections: bool = False
     only_modified: bool = False
+    combine_straight_imports: bool = False
     auto_identify_namespace_packages: bool = True
     namespace_packages: FrozenSet[str] = frozenset()
+    follow_links: bool = True
+    indented_import_headings: bool = True
+    honor_case_in_force_sorted_sections: bool = False
+    sort_relative_in_force_sorted_sections: bool = False
+    overwrite_in_place: bool = False
+    reverse_sort: bool = False
+    star_first: bool = False
+    import_dependencies = Dict[str, str]
 
     def __post_init__(self):
         py_version = self.py_version
@@ -230,6 +241,9 @@ class _Config:
                 self, "known_standard_library", frozenset(getattr(stdlibs, self.py_version).stdlib)
             )
 
+        if self.multi_line_output == WrapModes.VERTICAL_GRID_GROUPED_NO_COMMA:  # type: ignore
+            vertical_grid_grouped = WrapModes.VERTICAL_GRID_GROUPED  # type: ignore
+            object.__setattr__(self, "multi_line_output", vertical_grid_grouped)
         if self.force_alphabetical_sort:
             object.__setattr__(self, "force_alphabetical_sort_within_sections", True)
             object.__setattr__(self, "no_sections", True)
@@ -258,6 +272,8 @@ class Config(_Config):
     ):
         self._known_patterns: Optional[List[Tuple[Pattern[str], str]]] = None
         self._section_comments: Optional[Tuple[str, ...]] = None
+        self._skips: Optional[FrozenSet[str]] = None
+        self._skip_globs: Optional[FrozenSet[str]] = None
 
         if config:
             config_vars = vars(config).copy()
@@ -265,6 +281,8 @@ class Config(_Config):
             config_vars["py_version"] = config_vars["py_version"].replace("py", "")
             config_vars.pop("_known_patterns")
             config_vars.pop("_section_comments")
+            config_vars.pop("_skips")
+            config_vars.pop("_skip_globs")
             super().__init__(**config_vars)  # type: ignore
             return
 
@@ -383,7 +401,8 @@ class Config(_Config):
         for section in combined_config.get("sections", ()):
             if section in SECTION_DEFAULTS:
                 continue
-            elif not section.lower() in known_other:
+
+            if not section.lower() in known_other:
                 config_keys = ", ".join(known_other.keys())
                 warn(
                     f"`sections` setting includes {section}, but no known_{section.lower()} "
@@ -467,7 +486,7 @@ class Config(_Config):
         ext = ext.lstrip(".")
         if ext in self.supported_extensions:
             return True
-        elif ext in self.blocked_extensions:
+        if ext in self.blocked_extensions:
             return False
 
         # Skip editor backup files.
@@ -501,7 +520,7 @@ class Config(_Config):
             if file_path.name == ".git":  # pragma: no cover
                 return True
 
-            result = subprocess.run(  # nosec
+            result = subprocess.run(  # nosec # skipcq: PYL-W1510
                 ["git", "-C", str(file_path.parent), "check-ignore", "--quiet", os_path]
             )
             if result.returncode == 0:
@@ -511,7 +530,7 @@ class Config(_Config):
         if normalized_path[1:2] == ":":
             normalized_path = normalized_path[2:]
 
-        for skip_path in self.skip:
+        for skip_path in self.skips:
             if posixpath.abspath(normalized_path) == posixpath.abspath(
                 skip_path.replace("\\", "/")
             ):
@@ -519,11 +538,11 @@ class Config(_Config):
 
         position = os.path.split(file_name)
         while position[1]:
-            if position[1] in self.skip:
+            if position[1] in self.skips:
                 return True
             position = os.path.split(position[0])
 
-        for glob in self.skip_glob:
+        for glob in self.skip_globs:
             if fnmatch.fnmatch(file_name, glob) or fnmatch.fnmatch("/" + file_name, glob):
                 return True
 
@@ -564,6 +583,22 @@ class Config(_Config):
         self._section_comments = tuple(f"# {heading}" for heading in self.import_headings.values())
         return self._section_comments
 
+    @property
+    def skips(self) -> FrozenSet[str]:
+        if self._skips is not None:
+            return self._skips
+
+        self._skips = self.skip.union(self.extend_skip)
+        return self._skips
+
+    @property
+    def skip_globs(self) -> FrozenSet[str]:
+        if self._skip_globs is not None:
+            return self._skip_globs
+
+        self._skip_globs = self.skip_glob.union(self.extend_skip_glob)
+        return self._skip_globs
+
     def _parse_known_pattern(self, pattern: str) -> List[str]:
         """Expand pattern if identified as a directory and return found sub packages"""
         if pattern.endswith(os.path.sep):
@@ -639,7 +674,7 @@ def _find_config(path: str) -> Tuple[str, Dict[str, Any]]:
 def _get_config_data(file_path: str, sections: Tuple[str]) -> Dict[str, Any]:
     settings: Dict[str, Any] = {}
 
-    with open(file_path) as config_file:
+    with open(file_path, encoding="utf-8") as config_file:
         if file_path.endswith(".toml"):
             config = toml.load(config_file)
             for section in sections:
@@ -664,11 +699,15 @@ def _get_config_data(file_path: str, sections: Tuple[str]) -> Dict[str, Any]:
                 if section.startswith("*.{") and section.endswith("}"):
                     extension = section[len("*.{") : -1]
                     for config_key in config.keys():
-                        if config_key.startswith("*.{") and config_key.endswith("}"):
-                            if extension in map(
+                        if (
+                            config_key.startswith("*.{")
+                            and config_key.endswith("}")
+                            and extension
+                            in map(
                                 lambda text: text.strip(), config_key[len("*.{") : -1].split(",")
-                            ):
-                                settings.update(config.items(config_key))
+                            )
+                        ):
+                            settings.update(config.items(config_key))
 
                 elif config.has_section(section):
                     settings.update(config.items(section))
